# X.509 / TLS

# <i class="fa fa-user-secret" aria-hidden="true"></i>


## [X.509](https://fr.wikipedia.org/wiki/X.509)

* 1988 par l'UIT (agence de l'ONU le développement spécialisé dans les TIC )
* norme Pour
  * les certificats à clé publique
  * les listes de révocation de certificats
  * les attributs de certificats
  * un algorithme de validation du chemin de certification


## [X.509](https://fr.wikipedia.org/wiki/X.509)

* repose sur un système hiérarchique d'autorités de certification (AC)
  * une AC attribue un certificat liant
    * une clé publique
    * un nom distinctif (Distinguished Name -DN)
    * un nom alertnatif (Alternative Name - AN)
      * mail
      * enregistrement DNS


## [X.509](https://fr.wikipedia.org/wiki/X.509) Anatomie

* Version
* Serial
* Algorithme de signature du certificat
* **Subject** DN identifié par le certificat
* **Issuer** le signataire
* **Public Key**
* **Not Before** date de début de validité
* **Not After** date de finde validité
* Extensions
  * Key Usage
  * Subject Key Identifier
* Signature des informations ci-dessus par l'autorité de certification
  * hashé de tous les attributs
    * signé avec la clé privée de l'AC
    * signé avec la clé privée d'une AC de plus haut niveau


## X.509 Subject

Toujours un CN (common name)
et plein d'informations supplémentaires
notamment
pays
région
...

copier le prompt interactif d'une création de clé rsa / ssl


## X.509 générer un certificat

openssl req -x509 -key KEY -subj /CN=example.com -out CERTIFICATE

% cat CERTIFICATE
-----BEGIN CERTIFICATE-----
MIIC/zCCAeegAwIBAgIJAM4FANszQweWMA0GCSqGSIb3DQEBCwUAMBYxFDASBgNV
BAMMC2V4YW1wbGUuY29tMB4XDTE3MTAzMTExMzEzNFoXDTE3MTEzMDExMzEzNFow

* faire parler un certificat

openssl x509 -text -noout -in CERTIFICATE


## X.509 / RSA

* utilisé par openssl de manière transparente

* openssl genrsa -out KEY
  * contient la clé privée ET la clé publique

--copier le contenu du fichier

* openssl rsa -in KEY -pubout
  * permet d'extraire la partie publique uniquement

--copier le contenu de l'output


## X.509 vérification de la signature

openssl verify -CAfile CERTIFICATE CERTIFICATE

Certificat auto-signé
* Issuer et Subject identiques
  * X509v3 Subject Key Identifier = X509v3 Authority Key Identifier
* Tout le monde peut en fabriquer
* Rejeté par défaut par les navigateurs

## X.509 Autorité de certificaiton (CA)

* tiers de confiance
* recueillir les demandes de certifications
  * vérifie la validité de la demande
    * vérifie l'identité
      * preuve de controle des domaines
  * signent les certificats
  * gèrent les révocations


* Autorités de confiance  
  * importées par défaut dans le navigateur
* navigateur sans autorité de certifications importées?
* critères d'importation pour les navigateurs
  * infra ad hoc
  * payer (le navigateur)

CACERT n'est pas une autorité de certification intégré aux navigateurs


## Root CA

https://fr.wikipedia.org/wiki/Certificat_racine

es certificats racines sont des clés publiques non signées, ou auto-signées, dans lesquelles repose la confiance. Les logiciels, comme les navigateurs web ou les clients de messagerie détiennent des certificats racines de nombreuses autorités de certification commerciales ou gouvernementales. Quand un navigateur ouvre une connexion sécurisée (TLS/SSL) vers un site possédant un certificat émis par une autorité connue, il considère le site comme sûr dans la mesure où le chemin de certification est validé. Le passage en mode sécurisé est alors transparent.

## X.509 Demande de certificat Certificate Sign Request (CSR)

openssl req -new -subj /CN=example.com -key KEY -out CSR
% cat CSR

openssl req -in CSR -text -noout

CSR auto-signé
permet de valider l'intégrité du CSR
vérifie que le domaine correspond à un nom de domaine géré par le compte courant
* Serial
* Issuer
* validité
du CSR la CA créée le certificat, en ajoutant quelques informations et en signant avec sa clé privée
  * lui il a intérêt à bien gérer ses clés pour ne pas être compromis
    * en case de compromission on révoque

## X.509 Révocation de certificats
* Certificate Revocation List (CRL)
* contient une liste de certifact valide à révoquer
Mécanisme nécessaire en cas de compromission / décommissionnement
  ▶ On informe la CA
  ▶ La CA ajoute le certificat a sa liste de certificats révoqués
  ▶ Cette liste est signée par la CA
* Habituellement c’est la même clé qui signe les certificats et les CRL
* Quand-est-ce que le navigateur l’actualise ?


## Online Certificate Status Protocol (OCSP)
Protocole d’interogation de la validité d’un certificat
But : palier à la faiblesse de mise à jour des CRL par les clients
* [How can you set Firefox to, or tell if FF is always checking for certificate revocation?](https://support.mozilla.org/fr/questions/994310)
  * Interroger le répondeur OCSP pour confirmer la validité de vos certificats
* très peu déployé

## Résumé

des CA délivre des certificats
la confinance est centralisée sur la CA

## implémentations

### OpenSSL

* Implémenté en C
* Boîte à outils de chiffrement
  * bibliothèques cryptographie générale
  * bibliothèques implémentant le protocole SSL
  * commande en ligne
* Supporte SSL 2.0, SSL 3.0 et TLS 1.0.
* Distribué sous une licence de type Apache

<br />

### GnuTLS

* conforme aux spécifications de l'IETF
  * supporte TLS 1.1, TLS 1.0, SSL 3.0 et les extensions TLS
* permet l'authentification via les certificats X509 et PGP
* A la différence d'OpenSSL, GnuTLS est compatible avec les licences GPL


## SSL / TLS

* Crée un canal de communication **authentifié**, protégé en **confidentialité** et en **intégrité**
* Utilise des certificats X509
  * délivrés par des autorités de certification
* Utilise un système de chiffrement asymétrique
  * pour échanger une clé symétrique
* Protocle initialement pensé pour sécurisé HTTP
  * étendu à d'autres services ( SMTP, LDAP, VPN, etc ...)

<br />

#### Versions SSL (Secure Socket Layer)

* 1.0 par Netscape en 1994, pas de public release
* 2.0 par Netscape en Février 1995, (trous de sécurité) [The SSL Protocol Version 2.0](http://www.frameip.com/rfc/draftxxx.php)
* 3.0 par Netscape en Novembre 1996, [The SSL Protocol Version 3.0](http://www.frameip.com/rfc/draft302.php)

<br />

#### Versions TLS (Transport Layer Security)

* TLS 1.0 = SSL 3.1 [IETF](http://www.ietf.org/)
* 1.0 released en janvier 1999, [RFC 2246](http://www.frameip.com/rfc/rfc2246.php)
* 1.1 released en Avril 2006, [RFC 4346](http://www.frameip.com/rfc/rfc3546.php)
* 1.2 released en Août 2008, [RFC 5246](http://www.frameip.com/rfc/rfc4366.php)


## Connexion SSL/TLS (exemple : HTTPS)
1.Le serveur
▶ Envoie son certificat au client
2.Le client
▶ Reçoit un certificat
▶ Vérifie sa validité (domaine, date, émetteur, révocation)
▶ Génère une clé de chiffrement symétrique (secret partagée)
▶ Chiffre le secret partagée avec la clé publique du serveur
▶ Envoie la clé chiffrée au serveur (ClientKeyExchange)
3.Le serveur
▶ Reçoit la clé partagée chiffrée générée par le client
▶ Déchiffre la clé partagée avec sa clé privée

La suite de la communication est chhifrée symétriquement

## Connexion SSL/TLS (exemple : HTTPS)
Remarques :
▶ Le client a authentifié le serveur,
mais le serveur n’a aucune
information sur le client
  * possibilité d'avoir un certificat côté client

▶ Les paramètres de chiffrements
sont négociés et « jetables »

[slide55] mais reprendre les miens
détaillé les étapes

## en cas de compromission

* la clé privée est compromise
* elle permet donc déchiffrer ce qui a été chiffré avec la clé publique
* le secret de connexion est donc retrouvable à tous les coups


## Perfect Forward Secrecy (PFS)

* En français : confidentialité persistante
Si la clé privée du serveur est compromise, qu’en est-il des transactions passées ?
La PFS introduit un ServerKeyExchange
*  obligatoire pour la certification A+ (à vérifieir et creuser les catégories de certification).

* on voit passer que c'est du jaune
* mais ce ne donne pas d'avanatage
  * la couleur finale est impossible à trouver même en sachant qu'elle utilise du jaune

  * problème np complet (voir avec Pascal)


## Chiffrer pour plusieurs destinataires

On ne veut pas envoyer n messages distincts, ni un message chiffré avec  n clés différentes

1. On chiffre le message avec un algorithme symétrique et une clé aléatoire
2. On chiffre cette clé n fois pour les n destinataires
3. On envoie le message chiffré symétriquement et la clé partagée chiffrée pour chaque destinataire

Chaque destinataire peut déchiffrer la clé partagée avec sa clé privée et déchiffrer le message


## Chaînes de certification

* Un certificat est rarement signé par une CA racine
* La CA racine créé plusieurs CA intermédiaires
  * sous scellé / déconnecté / sorti pour générer
  * auto signé
* Les CA intermédiaires signent
  * les certificats des clients
  * d'autres CA intermédiaires
    * il faut alors fournir la chaîne de certification
      * au cas où l'intermédiaire ne soit pas dans le navigateur


## Certificats clients

* Un client peut présenter un certificat au serveur
* Le serveur vérifie si le certificat est signé par une CA de confiance
* Le serveur peut utiliser ces informations pour authentifier l’utilisateur
* Le certificat peut être stocké dans un périphérique (e.g. Yubikey), une carte à puce (e.g. CPS), ...
vérifier avec le CNRS

## Certification Authority Authorization (CAA)

* Une CA peut vérifier si elle est autorisée à émettre un certificat pour un domaine via le DNS (enregistrement CAA)

* Devenu obligatoire le 8 septembre 2017

* Le 9 septembre 2017, Comodo s’est fait pincer pour ne pas le respecter :
[Comodo Caught Breaking New CAA Standard One Day After It Went Into Effect](https://www.bleepingcomputer.com/news/security/comodo-caught-breaking-new-caa-standard-one-day-after-it-went-into-effect/)
(à vérifier masi c'est le cas ou un domaine est déjà enregistré chez un CA let's encrypt par exmple (il a l'enregsitrement) mais comodo n'en a pas tenu compte)

* Validation par les CA


## DNS-Based Authentication of Named Entities (DANE)

* Publication du certificat dans un enregistrement TLSA du DNS, protégé par DNSSEC

* Validation par les clients

On peut se passer des CA


## exemple des clés SSH

* exemple aussi avec github

## let's encrypt
